package com.sample.android.my_live_wallpaper1;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.service.wallpaper.WallpaperService;
import android.view.SurfaceHolder;
import android.view.MotionEvent;

public class My_Live_WallPaper1 extends WallpaperService {

	    //WallpaperService の生成の時に呼ばれる
	    public void onCreate()
	    {
	        super.onCreate();
	    }

	    //WallpaperService の破棄の時に呼ばれる
	    public void onDestroy()
	    {
	        super.onDestroy();
	    }

	    //WallPaper Engine の生成時に呼ばれる
	    public Engine onCreateEngine()
	    {
	        return new WallPaper_Engine1();  //描画用の自作の「WallPaper_Engine1」クラスのオブジェクトを返す
	    }

	    //ライブ壁紙の描画を行うEngineを継承した入れ子のクラス
	    public class WallPaper_Engine1 extends Engine implements Runnable
	    {
	    	/** 描画用のハンドラを用意 **/
	    	private final Handler handler = new Handler();

	        private Bitmap image1;   // イメージ1
	        private Bitmap image2;   // イメージ2
	        private Bitmap disp_image = null;
	        private Canvas canvas = null;

	        private boolean visible1;  //表示状態の変更用フラグ
	        private boolean touch_change = true; //Touchイベント用

	        private int    x = 10;   // X座標
	        private int    y = 10;   // Y座標
	        private int    dx = 10;  // X速度
	        private int    dy = 10;  // Y速度

	        private int    width1;
	        private int    height1;

	        private Thread  thread1;
	        private int touch_Action = -999; //タッチアクション

	        //コンストラクタ
	        public WallPaper_Engine1()
	        {
	            //resourceから表示する画像をロード
	            image1 = BitmapFactory.decodeResource(getResources(), R.drawable.rplane);
	            image2 = BitmapFactory.decodeResource(getResources(), R.drawable.lplane);
	            disp_image = BitmapFactory.decodeResource(getResources(), R.drawable.rplane);
	        }

	        //画面のタッチ
	        public void onTouchEvent(MotionEvent event)
	        {
	            super.onTouchEvent(event);  //タッチAction

	            touch_Action = event.getAction();

	            if(touch_Action == MotionEvent.ACTION_DOWN)
	            {
	          		if(touch_change == true)
	        		{
	        		   thread1 = new Thread(this);
	        		   thread1.start();  //Threadのスタート！

	        		   touch_change = false;
	        		}
	        		else
	        		{
	        		   thread1 = null;
	        		   touch_change = true;
	        		}
	            }
	        }

            public void run()
            {
                draw_Frame();  //自作の描画のメソッドを呼ぶ描画のメソッドを呼ぶ
            }

	        // Engineの生成時に呼び出される
	        public void onCreate(SurfaceHolder surfaceHolder)
	        {
	            super.onCreate(surfaceHolder);

	            setTouchEventsEnabled(true);  //TouchEvent OK
	        }

	        // Engineの破棄時に呼び出される
	        public void onDestroy()
	        {
	            super.onDestroy();
	            handler.removeCallbacks(thread1);  //Threadの停止
	        }

	        // 表示状態の変更(可視・不可視)時に呼び出される
	        public void onVisibilityChanged(boolean visible)
	        {
	            visible1 = visible;  //表示状態の変更用フラグ

	            if(visible1 == true )
	            {
	                draw_Frame();  //自作の描画のメソッドを呼ぶ描画のメソッドを呼ぶ
	            }else{
	                handler.removeCallbacks(thread1);  //Threadの停止
	            }
	        }

	        //Surfaceのサイズなどが変更されたときに呼び出されるメソッド
	        @Override
	        public void onSurfaceChanged(SurfaceHolder holder,int format, int width, int height)
	        {
	            super.onSurfaceChanged(holder, format, width, height);

	            width1 = width;    //this.width
	            height1 = height;  //this.height

	            draw_Frame();  //自作の描画のメソッドを呼ぶ
	        }

	        //Surfaceの生成時に呼び出される
	        public void onSurfaceCreated(SurfaceHolder holder)
	        {
	            super.onSurfaceCreated(holder);
	        }

	        //Surfaceの破棄時に呼び出される
	        public void onSurfaceDestroyed(SurfaceHolder holder)
	        {
	            super.onSurfaceDestroyed(holder);

	            visible1 = false;
	            handler.removeCallbacks(thread1 );  //Threadの停止
	        }

	        // オフセット変更時に呼び出される
	        public void onOffsetsChanged(float xOffset, float yOffset,float xStep, float yStep, int xPixels, int yPixels)
	        {
	            draw_Frame();  //自作の描画のメソッドを呼ぶ
	        }

	        // 自作のフレームの描画メソッド
	        private void draw_Frame()
	        {
	            final SurfaceHolder holder = getSurfaceHolder();

	            try{
	            	// SurfaceHolder から Canvasのオブジェクトを取得する
	                canvas = holder.lockCanvas(); // Canvasをロック

	                if (canvas != null)
	                {
	                    // Canvas(背景画面)の色
	                	canvas.drawColor(Color.YELLOW);
	                	//描画
	                	canvas.drawBitmap(disp_image, x, y, null);
	                }
	            }finally{

	                if (canvas != null) holder.unlockCanvasAndPost(canvas); // Canvasをアンロック
	            }

	            // 左端に達したとき、x方向の移動を反転
	            if( x < -50 )
	            {
	            	dx = -dx;
	                disp_image = image1;

	            }else if( width1 < x ) //右端に達したとき、x方向の移動を反転
	            {
	            	dx = -dx;
	            	disp_image = image2;
	            }

	            // 上端か下端に達したとき、y方向の移動を反転
	            if( y < 0 || height1 - image1.getHeight() < y )
	            {
	                dy = -dy;
	            }

	            x += dx; //移動
	            y += dy;

	            //再描画
	            handler.removeCallbacks(thread1 );

	            if(visible1 == true){ handler.postDelayed(thread1 , 100); }  //Threadの呼び出し時間の設定「100ms」
	        }
	    }
}